@@ -43,6 +43,7 @@ extern int preserve_gid;
 extern int preserve_times;
 extern int relative_paths;
 extern int copy_links;
+extern int remote_version;
 
 static char **local_exclude_list = NULL;
 
@@ -142,10 +143,14 @@ void send_file_entry_v11(struct file_struct *file,int f)
     write_int(f,(int)file->modtime);
   if (!(flags & SAME_MODE))
     write_int(f,(int)file->mode);
-  if (preserve_uid && !(flags & SAME_UID))
-    write_int(f,(int)file->uid);
-  if (preserve_gid && !(flags & SAME_GID))
-    write_int(f,(int)file->gid);
+  if (preserve_uid && !(flags & SAME_UID)) {
+	  add_uid(file->uid);
+	  write_int(f,(int)file->uid);
+  }
+  if (preserve_gid && !(flags & SAME_GID)) {
+	  add_gid(file->gid);
+	  write_int(f,(int)file->gid);
+  }
   if (preserve_devices && IS_DEVICE(file->mode) && !(flags & SAME_RDEV))
     write_int(f,(int)file->rdev);
 
@@ -373,14 +378,23 @@ static void send_directory(int f,struct file_list *flist,char *dir)
   fname[MAXPATHLEN-1]=0;
   l = strlen(fname);
   if (fname[l-1] != '/') {
+        if (l == MAXPATHLEN-1) {
+              fprintf(FERROR,"skipping long-named directory %s\n",fname);
+              closedir(d);
+              return;
+        }
 	  strcat(fname,"/");
 	  l++;
   }
   p = fname + strlen(fname);
 
   if (cvs_exclude) {
-    strcpy(p,".cvsignore");
-    local_exclude_list = make_exclude_list(fname,NULL,0);
+    if (strlen(fname) + strlen(".cvsignore") <= MAXPATHLEN-1) {
+      strcpy(p,".cvsignore");
+      local_exclude_list = make_exclude_list(fname,NULL,0);
+    } else {
+      fprintf(FERROR,"cannot cvs-exclude in long-named directory %s\n",fname);
+    }
   }  
 
   for (di=readdir(d); di; di=readdir(d)) {
@@ -404,7 +418,7 @@ struct file_list *send_file_list(int f,int argc,char *argv[])
   char dbuf[MAXPATHLEN];
   struct file_list *flist;
 
-  if (verbose && recurse && !am_server) {
+  if (verbose && recurse && !am_server && f != -1) {
     fprintf(FINFO,"building file list ... ");
     fflush(FINFO);
   }
@@ -488,11 +502,16 @@ struct file_list *send_file_list(int f,int argc,char *argv[])
     write_flush(f);
   }
 
-  if (verbose && recurse && !am_server)
+  if (verbose && recurse && !am_server && f != -1)
     fprintf(FINFO,"done\n");
 
   clean_flist(flist);
 
+  /* now send the uid/gid list. This was introduced in protocol version 15 */
+  if (f != -1 && remote_version >= 15) {
+	  send_uid_list(f);
+  }
+
   return flist;
 }
 
@@ -555,6 +574,11 @@ struct file_list *recv_file_list(int f)
     fprintf(FINFO,"done\n");
   }
 
+  /* now recv the uid/gid list. This was introduced in protocol version 15 */
+  if (f != -1 && remote_version >= 15) {
+	  recv_uid_list(f, flist);
+  }
+
   return flist;
 
 oom:
@@ -572,16 +596,6 @@ int file_compare(struct file_struct *f1,struct file_struct *f2)
 }
 
 
-/* we need this function because of the silly way in which duplicate
-   entries are handled in the file lists - we can't change this
-   without breaking existing versions */
-static int flist_up(struct file_list *flist, int i)
-{
-	while (!flist->files[i].name) i++;
-	return i;
-}
-
-
 int flist_find(struct file_list *flist,struct file_struct *f)
 {
 	int low=0,high=flist->count-1;
