@@ -1,4 +1,4 @@
-/*	$Id: zlib.h,v 1.1 1996/07/03 04:05:49 paulus Exp $	*/
+/*	$Id: zlib.h,v 1.3 1997/12/18 11:18:35 tridge Exp $	*/
 
 /*
  * This file is derived from zlib.h and zconf.h from the zlib-0.95
@@ -112,7 +112,8 @@
 
 typedef unsigned char  Byte;  /* 8 bits */
 typedef unsigned int   uInt;  /* 16 bits or more */
-typedef unsigned long  uLong; /* 32 bits or more */
+typedef int32 Long; /* 32 bits or more */
+typedef uint32 uLong; /* 32 bits or more */
 
 typedef Byte FAR Bytef;
 typedef char FAR charf;
