@@ -26,6 +26,7 @@ off_t total_size = 0;
 int block_size=BLOCK_SIZE;
 
 char *backup_suffix = BACKUP_SUFFIX;
+char *tmpdir = NULL;
 
 static char *rsync_path = RSYNC_NAME;
 
@@ -52,6 +53,7 @@ int do_compression=0;
 int am_root=0;
 int orig_umask=0;
 int relative_paths=0;
+int numeric_ids = 0;
 
 extern int csum_length;
 
@@ -158,9 +160,22 @@ static void server_options(char **args,int *argc)
     args[ac++] = bsize;
   }    
 
+  if (strcmp(backup_suffix, BACKUP_SUFFIX)) {
+	  args[ac++] = "--suffix";
+	  args[ac++] = backup_suffix;
+  }
+
   if (delete_mode)
     args[ac++] = "--delete";
 
+  if (numeric_ids)
+    args[ac++] = "--numeric-ids";
+
+  if (tmpdir) {
+	  args[ac++] = "--temp-dir";
+	  args[ac++] = tmpdir;
+  }
+
   *argc = ac;
 }
 
@@ -429,7 +444,9 @@ static void usage(FILE *f)
   fprintf(f,"    --rsync-path PATH    specify path to rsync on the remote machine\n");
   fprintf(f,"-C, --cvs-exclude        auto ignore files in the same way CVS does\n");
   fprintf(f,"    --delete             delete files that don't exist on the sending side\n");
+  fprintf(f,"    --numeric-ids        don't map uid/gid values by user/group name\n");
   fprintf(f,"-I, --ignore-times       don't exclude files that match length and time\n");
+  fprintf(f,"-T  --temp-dir DIR       create temporary files in directory DIR\n");
   fprintf(f,"-z, --compress           compress file data\n");
   fprintf(f,"    --exclude FILE       exclude file FILE\n");
   fprintf(f,"    --exclude-from FILE  exclude files listed in FILE\n");
@@ -442,15 +459,16 @@ static void usage(FILE *f)
 }
 
 enum {OPT_VERSION,OPT_SUFFIX,OPT_SENDER,OPT_SERVER,OPT_EXCLUDE,
-      OPT_EXCLUDE_FROM,OPT_DELETE,OPT_RSYNC_PATH};
+      OPT_EXCLUDE_FROM,OPT_DELETE,OPT_NUMERIC_IDS,OPT_RSYNC_PATH};
 
-static char *short_options = "oblLWHpguDCtcahvrRIxnSe:B:z";
+static char *short_options = "oblLWHpguDCtcahvrRIxnSe:B:T:z";
 
 static struct option long_options[] = {
   {"version",     0,     0,    OPT_VERSION},
   {"server",      0,     0,    OPT_SERVER},
   {"sender",      0,     0,    OPT_SENDER},
   {"delete",      0,     0,    OPT_DELETE},
+  {"numeric-ids", 0,     0,    OPT_NUMERIC_IDS},
   {"exclude",     1,     0,    OPT_EXCLUDE},
   {"exclude-from",1,     0,    OPT_EXCLUDE_FROM},
   {"rsync-path",  1,     0,    OPT_RSYNC_PATH},
@@ -479,6 +497,7 @@ static struct option long_options[] = {
   {"rsh",         1,     0,    'e'},
   {"suffix",      1,     0,    OPT_SUFFIX},
   {"block-size",  1,     0,    'B'},
+  {"temp-dir",    1,     0,    'T'},
   {"compress",	  0,	 0,    'z'},
   {0,0,0,0}};
 
@@ -544,6 +563,10 @@ int main(int argc,char *argv[])
 	  delete_mode = 1;
 	  break;
 
+	case OPT_NUMERIC_IDS:
+	  numeric_ids = 1;
+	  break;
+
 	case OPT_EXCLUDE:
 	  add_exclude(optarg);
 	  break;
@@ -667,6 +690,10 @@ int main(int argc,char *argv[])
 	  block_size = atoi(optarg);
 	  break;
 
+	case 'T':
+		tmpdir = optarg;
+		break;
+
         case 'z':
 	  do_compression = 1;
 	  break;
