@@ -29,6 +29,7 @@ extern time_t starttime;
 extern int remote_version;
 
 extern char *backup_suffix;
+extern char *tmpdir;
 
 extern int whole_file;
 extern int block_size;
@@ -252,13 +253,36 @@ static int set_perms(char *fname,struct file_struct *file,struct stat *st,
 }
 
 
+/* choose whether to skip a particular file */
+static int skip_file(char *fname,
+		     struct file_struct *file, struct stat *st)
+{
+	if (st->st_size != file->length) {
+		return 0;
+	}
+	
+	/* if always checksum is set then we use the checksum instead 
+	   of the file time to determine whether to sync */
+	if (always_checksum && S_ISREG(st->st_mode)) {
+		char sum[MD4_SUM_LENGTH];
+		file_checksum(fname,sum,st->st_size);
+		return (memcmp(sum,file->sum,csum_length) == 0);
+	}
+
+	if (ignore_times) {
+		return 0;
+	}
+
+	return (st->st_mtime == file->modtime);
+}
+
+
 void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
 {  
   int fd;
   struct stat st;
   struct map_struct *buf;
   struct sum_struct *s;
-  char sum[MD4_SUM_LENGTH];
   int statret;
   struct file_struct *file = &flist->files[i];
 
@@ -382,20 +406,13 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
     return;
   }
 
-  if (update_only && st.st_mtime >= file->modtime) {
+  if (update_only && st.st_mtime > file->modtime) {
     if (verbose > 1)
       fprintf(FERROR,"%s is newer\n",fname);
     return;
   }
 
-  if (always_checksum && S_ISREG(st.st_mode)) {
-    file_checksum(fname,sum,st.st_size);
-  }
-
-  if (st.st_size == file->length &&
-      ((!ignore_times && st.st_mtime == file->modtime) ||
-       (always_checksum && S_ISREG(st.st_mode) && 	  
-	memcmp(sum,file->sum,csum_length) == 0))) {
+  if (skip_file(fname, file, &st)) {
     set_perms(fname,file,&st,1);
     return;
   }
@@ -535,6 +552,61 @@ static void delete_one(struct file_struct *f)
 }
 
 
+/* yuck! This function wouldn't have been necessary if I had the sorting
+   algorithm right. Unfortunately fixing the sorting algorithm would introduce
+   a backward incompatibility as file list indexes are sent over the link.
+
+   The aim is to see if a directory has already had the deletion algorithm applied
+   to it (due to recursion), and if so to skip it. The bisection is to 
+   prevent this being an n^2 algorithm */
+static int delete_already_done(struct file_list *flist,int j)
+{
+	int low=0,high=j-1;
+	char *name;
+	char *p;
+
+	if (j == 0) return 0;
+
+	name = strdup(flist->files[j].name);
+
+	if (!name) {
+		fprintf(FERROR,"out of memory in delete_already_done");
+		exit_cleanup(1);
+	}
+
+	p = strrchr(name,'/');
+	if (!p) {
+		free(name);
+		return 0;
+	}
+	*p = 0;
+
+	while (low != high) {
+		int mid = (low+high)/2;
+		int ret = strcmp(flist->files[flist_up(flist, mid)].name,name);
+		if (ret == 0) {
+			free(name);
+			return 1;
+		}
+		if (ret > 0) {
+			high=mid;
+		} else {
+			low=mid+1;
+		}
+	}
+
+	low = flist_up(flist, low);
+
+	if (strcmp(flist->files[low].name,name) == 0) {
+		free(name);
+		return 1;
+	}
+
+	free(name);
+	return 0;
+}
+
+
 /* this deletes any files on the receiving side that are not present
    on the sending side. For version 1.6.4 I have changed the behaviour
    to match more closely what most people seem to expect of this option */
@@ -542,23 +614,22 @@ static void delete_files(struct file_list *flist)
 {
   struct file_list *local_file_list;
   int i, j;
-  char *last_name=NULL;
 
   if (cvs_exclude)
     add_cvs_excludes();
 
   for (j=0;j<flist->count;j++) {
+	  char *name = flist->files[j].name;
+
 	  if (!S_ISDIR(flist->files[j].mode)) continue;
-	  if (strcmp(flist->files[j].name,".")==0) continue;
-	  if (last_name &&
-	      flist->files[j].name[strlen(last_name)] == '/' &&
-	      strncmp(flist->files[j].name,last_name, strlen(last_name))==0)
-		  continue;
-	  last_name = flist->files[j].name;
-	  if (!(local_file_list = send_file_list(-1,1,&last_name)))
+
+	  if (delete_already_done(flist, j)) continue;
+
+	  if (!(local_file_list = send_file_list(-1,1,&name)))
 		  continue;
+
 	  if (verbose > 1)
-		  fprintf(FINFO,"deleting in %s\n", last_name);
+		  fprintf(FINFO,"deleting in %s\n", name);
 
 	  for (i=local_file_list->count-1;i>=0;i--) {
 		  if (!local_file_list->files[i].name) continue;
@@ -674,7 +745,17 @@ int recv_files(int f_in,struct file_list *flist,char *local_name,int f_gen)
 	close(fd1);
 	continue;
       }
-      sprintf(fnametmp,"%s.XXXXXX",fname);
+      if (tmpdir) {
+	      char *f;
+	      f = strrchr(fname,'/');
+	      if (f == NULL) 
+		      f = fname;
+	      else 
+		      f++;
+	      sprintf(fnametmp,"%s/%s.XXXXXX",tmpdir,f);
+      } else {
+	      sprintf(fnametmp,"%s.XXXXXX",fname);
+      }
       if (NULL == mktemp(fnametmp)) {
 	fprintf(FERROR,"mktemp %s failed\n",fnametmp);
 	receive_data(f_in,buf,-1,NULL);
@@ -682,10 +763,10 @@ int recv_files(int f_in,struct file_list *flist,char *local_name,int f_gen)
 	close(fd1);
 	continue;
       }
-      fd2 = open(fnametmp,O_WRONLY|O_CREAT,file->mode);
+      fd2 = open(fnametmp,O_WRONLY|O_CREAT|O_EXCL,file->mode);
       if (fd2 == -1 && relative_paths && errno == ENOENT && 
 	  create_directory_path(fnametmp) == 0) {
-	      fd2 = open(fnametmp,O_WRONLY|O_CREAT,file->mode);
+	      fd2 = open(fnametmp,O_WRONLY|O_CREAT|O_EXCL,file->mode);
       }
       if (fd2 == -1) {
 	fprintf(FERROR,"open %s : %s\n",fnametmp,strerror(errno));
@@ -727,14 +808,27 @@ int recv_files(int f_in,struct file_list *flist,char *local_name,int f_gen)
 
       /* move tmp file over real file */
       if (rename(fnametmp,fname) != 0) {
-	fprintf(FERROR,"rename %s -> %s : %s\n",
-		fnametmp,fname,strerror(errno));
-	unlink(fnametmp);
+	      if (errno == EXDEV) {
+		      /* rename failed on cross-filesystem link.  
+			 Copy the file instead. */
+		      if (copy_file(fnametmp,fname, file->mode)) {
+			      fprintf(FERROR,"copy %s -> %s : %s\n",
+				      fnametmp,fname,strerror(errno));
+		      } else {
+			      set_perms(fname,file,NULL,0);
+		      }
+		      unlink(fnametmp);
+	      } else {
+		      fprintf(FERROR,"rename %s -> %s : %s\n",
+			      fnametmp,fname,strerror(errno));
+		      unlink(fnametmp);
+	      }
+      } else {
+	      set_perms(fname,file,NULL,0);
       }
 
       cleanup_fname = NULL;
 
-      set_perms(fname,file,NULL,0);
 
       if (!recv_ok) {
 	if (verbose > 1)
@@ -803,6 +897,11 @@ off_t send_files(struct file_list *flist,int f_out,int f_in)
       if (file->dir) {
 	strncpy(fname,file->dir,MAXPATHLEN-1);
 	fname[MAXPATHLEN-1] = 0;
+      if (strlen(fname) == MAXPATHLEN-1) {
+        fprintf(FERROR, "send_files failed on long-named directory %s\n",
+                fname);
+        return -1;
+      }
 	strcat(fname,"/");
       }
       strncat(fname,file->name,MAXPATHLEN-strlen(fname));
