@@ -39,7 +39,7 @@
 #define SAME_TIME (1<<7)
 
 /* update this if you make incompatible changes */
-#define PROTOCOL_VERSION 14
+#define PROTOCOL_VERSION 15
 #define MIN_PROTOCOL_VERSION 10
 #define MAX_PROTOCOL_VERSION 20
 
@@ -163,6 +163,10 @@
 #include "lib/getopt.h"
 #endif
 
+/* these are needed for the uid/gid mapping code */
+#include <pwd.h>
+#include <grp.h>
+
 #ifndef S_IFLNK
 #define S_IFLNK  0120000
 #endif
@@ -248,6 +252,16 @@ struct map_struct {
   int fd,size,p_size,p_offset,p_len;
 };
 
+/* we need this function because of the silly way in which duplicate
+   entries are handled in the file lists - we can't change this
+   without breaking existing versions */
+static int flist_up(struct file_list *flist, int i)
+{
+	while (!flist->files[i].name) i++;
+	return i;
+}
+
+
 #include "byteorder.h"
 #include "version.h"
 #include "proto.h"
